package com.chelvc.cloud.admin.controller;

import java.util.List;
import javax.validation.Valid;
import javax.validation.constraints.Min;

import com.chelvc.cloud.uc.api.dto.DepartmentDTO;
import com.chelvc.cloud.uc.api.param.DepartmentModifyParam;
import com.chelvc.cloud.uc.api.param.DepartmentQueryParam;
import com.chelvc.cloud.uc.api.service.DepartmentService;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.util.ResourceUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 组织接口
 *
 * @author Woody
 * @date 2021/10/20
 */
@Validated
@RestController
@ResponseWrapping
@PreAuthorize("isScope('EMPLOYEE')")
public class DepartmentController {
    @DubboReference
    private DepartmentService departmentService;

    /**
     * 新增部门
     *
     * @param param 新增参数
     * @return 部门主键
     */
    @PostMapping("/department")
    public Long addDepartment(@RequestBody @Valid DepartmentModifyParam param) {
        return this.departmentService.addDepartment(param);
    }

    /**
     * 修改部门
     *
     * @param id    部门主键
     * @param param 修改参数
     */
    @PutMapping("/department/{id}")
    public void updateDepartment(@PathVariable("id") @Min(value = 1, message = "部门主键不能小于1") Long id,
                                 @RequestBody @Valid DepartmentModifyParam param) {
        this.departmentService.updateDepartment(id, param);
    }

    /**
     * 获取部门信息
     *
     * @param id 部门主键
     * @return 部门信息
     */
    @GetMapping("/department/{id}")
    public DepartmentDTO getDepartment(@PathVariable("id") @Min(value = 1, message = "部门主键不能小于1") Long id) {
        return ResourceUtils.required(this.departmentService.getDepartment(id), "部门不存在");
    }

    /**
     * 查询部门树列表
     *
     * @param param 查询参数
     * @return 部门树列表
     */
    @GetMapping("/department/trees")
    public List<DepartmentDTO> listDepartmentTrees(@Valid DepartmentQueryParam param) {
        return this.departmentService.listDepartmentTrees(param);
    }
}
